@@ -1,4 +1,4 @@
-﻿// -*- mode: c++; c-file-style: "linux"; c-basic-offset: 2; indent-tabs-mode: nil -*-
+// -*- mode: c++; c-file-style: "linux"; c-basic-offset: 2; indent-tabs-mode: nil -*-
 //
 //  Copyright (C) 2004-2015 Andrej Vodopivec <andrej.vodopivec@gmail.com>
 //            (C) 2012 Doug Ilijev <doug.ilijev@gmail.com>
@@ -16,7 +16,7 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 //
 //  SPDX-License-Identifier: GPL-2.0+
 
@@ -29,10 +29,15 @@ dialog. The preferences themself will be read directly using
 <code> config->Read </code>, instead, where needed or from Configuration.
 */
 
+extern unsigned int view_refresh_svg_gz_len;
+extern unsigned char view_refresh_svg_gz[];
+
+#include "precomp.h"
 #include <wx/wx.h>
 #include <wx/image.h>
+#include <wx/grid.h>
 #include <wx/hashmap.h>
-
+#include <memory>
 #include <wx/propdlg.h>
 #include <wx/generic/propdlg.h>
 #include <wx/spinctrl.h>
@@ -55,8 +60,8 @@ enum
   checkbox_bold,
   checkbox_italic,
   checkbox_underlined,
+  button_defaultFont,
   button_mathFont,
-  font_family,
   style_font_family,
   language_id,
   save_id,
@@ -74,11 +79,20 @@ class ConfigDialogue : public wxPropertySheetDialog
 {
 public:
   //! The constructor
-  ConfigDialogue(wxWindow *parent, Configuration *config);
+  ConfigDialogue(wxWindow *parent, Configuration *cfg);
 
   //! The destructor
   ~ConfigDialogue();
 
+  //! The export formats we support for HTML equations
+  enum htmlExportFormats
+  {
+    mathJaX_TeX = 0,
+    bitmap = 1,
+    mathML_mathJaX = 2,
+    svg = 3
+  };
+
   /*! Called if the color of an item has been changed
 
     called from class ColorPanel
@@ -93,10 +107,30 @@ public:
   void WriteSettings();
 
 private:
+  enum newVariables
+  {
+    MAXIMA_DEFAULT_LISP,
+    MAXIMA_IMAGESDIR,
+    MAXIMA_USERDIR,
+    MAXIMA_DIRECTORY,
+    MAXIMA_TEMPDIR,
+    MAXIMA_OBJDIR,
+    MAXIMA_DOC_PREFIX,
+    GCL_GC_PAGE_THRESH,
+    GCL_GC_ALLOC_MIN,
+    GCL_GC_PAGE_MAX,
+    GCL_MEM_MULTIPLE,
+    GCL_MULTIPROCESS_MEMORY_POOL,
+    LANG,
+    HOME,
+    VAR_DELETE
+  };
+  std::unique_ptr<struct NSVGrasterizer, decltype(std::free)*> m_svgRast{nullptr, std::free};
   //! The configuration storage
   Configuration *m_configuration;
   
-  WX_DECLARE_STRING_HASH_MAP(int, Languages);
+  WX_DECLARE_STRING_HASH_MAP(wxString, StringHash);
+  WX_DECLARE_STRING_HASH_MAP(long, Languages);
   Languages m_languages;
   /*! TheSample text that is shown by the style selector.
 
@@ -107,30 +141,15 @@ private:
   public:
     //! The constructor
     ExamplePanel(wxWindow *parent, int id, wxPoint pos, wxSize size) : wxPanel(parent, id, pos, size)
-      {
-#if defined (__WXOSX__)
-        m_size = 12;
-#else
-        m_size = 10;
-#endif
-        m_italic = false;
-        m_bold = false;
-        m_underlined = false;
-      };
+    { Connect(wxEVT_PAINT, wxPaintEventHandler(ConfigDialogue::ExamplePanel::OnPaint)); };
 
-    //! Sets all user-changable elements of style of the example at once.
-    void SetStyle(wxColour fg_color, bool italic, bool bold, bool underlined, wxString font)
-      {
-        m_fgColor = fg_color;
-        m_italic = italic;
-        m_bold = bold;
-        m_underlined = underlined;
-        m_font = font;
-      }
-
-    //! Sets the font size of the example
-    void SetFontSize(int size)
-      { m_size = size; }
+    //! Sets the text style of the example
+    void SetStyle(const Style &style)
+    {
+      if (m_style.IsStyleEqualTo(style)) return;
+      m_style = style;
+      Refresh();
+    }
 
   private:
     /*! Actually updates the formatting example
@@ -139,19 +158,8 @@ private:
     */
     void OnPaint(wxPaintEvent &event);
 
-    //! The foreground color of the currently selected item type
-    wxColour m_fgColor;
-    //! Is the currently selected item type displayed in italic?
-    bool m_italic;
-    //! Is the currently selected item type displayed in bold?
-    bool m_bold;
-    //! Is the currently selected item type displayed underlined?
-    bool m_underlined;
-    //! The font the currently selected item type is displayed with
-    wxString m_font;
-    //! The size of the characters of the currently selected item type
-    int m_size;
-    DECLARE_EVENT_TABLE()
+    //! The text style of this example
+    Style m_style;
   };
 
   /*! A rectangle showing the color of an item
@@ -176,7 +184,6 @@ private:
   private:
     ConfigDialogue *m_configDialogue;
     wxColor m_color;
-    DECLARE_EVENT_TABLE()
   };
 
 
@@ -184,18 +191,20 @@ private:
 
     This method sets the window title, the tool tips etc.
    */
-  void SetProperties();
+  void SetCheckboxValues();
 
   //! Calculates the size of the images for a configuration tab
   int GetImageSize();
 
   //! Loads the image for a configuration tab
   wxBitmap GetImage(wxString name,
-                   unsigned char *data_128, size_t len_128,
-                   unsigned char *data_192, size_t len_192);
+                   unsigned char *data, size_t len);
 
   //! The panel that allows to choose which formats to put on the clipboard
-  wxPanel *CreateClipboardPanel();
+  wxWindow *CreateClipboardPanel();
+
+  //! The panel that allows to choose which formats to put on the clipboard
+  wxWindow *CreateRevertToDefaultsPanel();
 
   wxCheckBox *m_copyBitmap, *m_copyMathML, *m_copyMathMLHTML, *m_copyRTF, *m_copySVG;
   #if wxUSE_ENH_METAFILE
@@ -203,36 +212,52 @@ private:
   #endif
 
   //! The panel that allows to set the editing options
-  wxPanel *CreateWorksheetPanel();
+  wxWindow *CreateWorksheetPanel();
 
   //! A panel that allows to set general options
-  wxPanel *CreateOptionsPanel();
+  wxWindow *CreateOptionsPanel();
 
   //! The panel that allows to set options affecting the export functionality
-  wxPanel *CreateExportPanel();
+  wxWindow *CreateExportPanel();
 
   //! The panel that allows to change styles
-  wxPanel *CreateStylePanel();
+  wxWindow *CreateStylePanel();
 
   //! The panel that allows to change maxima-specific configurations.
-  wxPanel *CreateMaximaPanel();
+  wxWindow *CreateMaximaPanel();
 
   //! The panel that allows to specify startup commands
-  wxPanel *CreateStartupPanel();
-
-  // end wxGlade
+  wxWindow *CreateStartupPanel();
+  
 protected:
-  //! The name of maxima's startup file.
+  wxGrid *m_maximaEnvVariables;
+  void OnImport(wxCommandEvent& event);
+  static void CopyConfig(wxConfigBase *src, wxConfigBase *dst, wxString dir = wxT("/"));
+  void OnReloadAll(wxCommandEvent& event);
+  void OnReloadStyles(wxCommandEvent& event);
+  void OnResetAllToDefaults(wxCommandEvent& event);
+  void OnExportAll(wxCommandEvent& event);
+  void OnResetStyles(wxCommandEvent& event);
+  void OnChangeMaximaEnvVar(wxGridEvent& event);
+  void OnMaximaEnvRightClick(wxGridEvent& event);
+  void OnMouseMotion_MaximaEnv(wxMouseEvent &event);
+  void OnNewEnvMenu(wxCommandEvent &event);
+  void OnClickMaximaEnvVal(int row);
+  void OnChangeMaximaCellClick(wxGridEvent& event);
+
+//! The name of maxima's startup file.
   wxString m_startupFileName;
   //! The name of wxMaxima's startup file.
   wxString m_wxStartupFileName;
   //! The text "Maxima Program" that can change color.
   wxStaticText *m_mp;
-
+  StringHash m_maximaEnvDoc;
   //! Autodetect the maxima location?
   wxRadioButton *m_autodetectMaxima;
   //! The radio button that is set if m_autodetectMaxima is unset
   wxRadioButton *m_noAutodetectMaxima;
+  wxRadioButton *m_autodetectHelpBrowser;
+  wxRadioButton *m_noAutodetectHelpBrowser;
   //! Autodetect the mathJaX location?
   wxRadioButton *m_autodetectMathJaX;
   //! The radio button that is set if m_autodetectMathJaX is unset
@@ -249,27 +274,27 @@ protected:
   //! A textbox containing wxMaxima's startup commands
   wxTextCtrl *m_wxStartupCommands;
   wxTextCtrl *m_maximaUserLocation;
+  wxTextCtrl *m_helpBrowserUserLocation;
   wxTextCtrl *m_documentclass;
+  wxTextCtrl *m_documentclassOptions;
   wxTextCtrl *m_texPreamble;
-  wxSpinCtrl *m_autoSaveInterval;
+  wxCheckBox *m_autoSave;
   wxButton *m_mpBrowse;
   wxTextCtrl *m_additionalParameters;
   wxTextCtrl *m_mathJaxURL;
   wxChoice *m_language;
   wxTextCtrl *m_symbolPaneAdditionalChars;
-  wxCheckBox *m_saveSize;
   wxCheckBox *m_abortOnError;
+  wxCheckBox *m_offerKnownAnswers;
   wxCheckBox *m_restartOnReEvaluation;
   wxCheckBox *m_wrapLatexMath;
-  wxCheckBox *m_savePanes;
-  wxCheckBox *m_usepngCairo;
+  wxCheckBox *m_usesvg;
   wxCheckBox *m_antialiasLines;
   wxSpinCtrl *m_defaultFramerate;
   wxSpinCtrl *m_defaultPlotWidth;
   wxSpinCtrl *m_defaultPlotHeight;
   wxSpinCtrl *m_displayedDigits;
-  //! A checkbox that allows to select if the LaTeX file should contain animations.
-  wxCheckBox *m_AnimateLaTeX;
+  wxSpinCtrl *m_maxClipbrdBitmapMegabytes;
   //! A checkbox that asks if TeX should put the exponents above or after the subscripts.
   wxCheckBox *m_TeXExponentsAfterSubscript;
   //! A checkbox that asks if TeX should use the \\partial symbol for representing diff()
@@ -281,7 +306,9 @@ protected:
   wxCheckBox *m_matchParens;
   wxChoice *m_showLength;
   wxChoice *m_autosubscript;
-  wxCheckBox *m_enterEvaluates;
+  wxRadioButton *m_enterEvaluates;
+  wxRadioButton *m_ctrlEnterEvaluates;
+  wxCheckBox *m_numpadEnterEvaluates;
   wxCheckBox *m_saveUntitled;
   wxCheckBox *m_openHCaret;
   wxCheckBox *m_insertAns;
@@ -299,9 +326,9 @@ protected:
   wxCheckBox *m_incrementalSearch;
   wxCheckBox *m_notifyIfIdle;
   wxChoice *m_showUserDefinedLabels;
-  wxButton *m_getFont;
+  wxButton *m_getDefaultFont;
+  wxButton *m_getMathFont;
   wxButton *m_getStyleFont;
-  wxFontEncoding m_fontEncoding;
   wxListBox *m_styleFor;
   //! An example rectangle with the font color
   ColorPanel *m_styleColor;
@@ -311,23 +338,22 @@ protected:
   wxCheckBox *m_fixedFontInTC;
   wxCheckBox *m_unixCopy;
   wxCheckBox *m_changeAsterisk;
-  wxCheckBox *m_useJSMath;
+  wxCheckBox *m_hidemultiplicationSign;
+  wxCheckBox *m_latin2Greek;
   wxCheckBox *m_useUnicodeMaths;
   wxCheckBox *m_keepPercentWithSpecials;
   wxBookCtrlBase *m_notebook;
-  wxStaticText *m_mathFont;
-  wxButton *m_getMathFont;
-  wxString m_mathFontName;
   wxButton *m_saveStyle, *m_loadStyle;
   wxSpinCtrl *m_defaultPort;
   ExamplePanel *m_examplePanel;
-  // end wxGlade
-
+  wxSpinCtrl *m_maxGnuplotMegabytes;
+  wxTextCtrl *m_autoMathJaxURL;
+  int m_maximaEmvRightClickRow = 0;
   //! Is called when the path to the maxima binary was changed.
   void MaximaLocationChanged(wxCommandEvent &unused);
 
   //! Is called when the path to the maxima binary was changed.
-  void UsepngcairoChanged(wxCommandEvent &event);
+  void UsesvgChanged(wxCommandEvent &event);
 
   //! Is called when the configuration dialog is closed.
   void OnClose(wxCloseEvent &event);
@@ -335,10 +361,12 @@ protected:
   //! Starts the file chooser that allows selecting where the maxima binary lies
   void OnMpBrowse(wxCommandEvent &event);
 
+  void OnHelpBrowserBrowse(wxCommandEvent&  event);
+
   void OnIdle(wxIdleEvent &event);
 
-  //! Starts the font selector dialog for the math font
-  void OnMathBrowse(wxCommandEvent &event);
+  //! Starts the font selector dialog triggered by the math or default font buttons
+  void OnFontButton(wxCommandEvent &event);
 
   //! Called if a new item type that is to be styled is selected
   void OnChangeStyle(wxCommandEvent &event);
@@ -358,14 +386,23 @@ protected:
   //! A "export the configuration" dialog
   void LoadSave(wxCommandEvent &event);
 
+  //! Map the style list index to a style
+  static TextStyle StyleForListIndex(int index);
+  //! Map the style to the style list index
+  static int StyleListIndexForStyle(TextStyle style);
+  //! Get the style currently selected in the m_styleFor control
+  TextStyle GetSelectedStyle() const;
+
+  //! Sets the label for the font setting button given by the style (either TS_DEFAULT or TS_MATH)
+  void UpdateButton(TextStyle style);
+
   //! The size of the text font
   int m_fontSize;
   //! The size of the maths font.
   int m_mathFontSize;
 
   //! A list containing the pictograms for the tabs.
-  wxImageList *m_imageList;
-DECLARE_EVENT_TABLE()
+  std::unique_ptr<wxImageList> m_imageList;
 };
 
 #ifndef __WXMSW__
