@@ -1,4 +1,4 @@
-﻿// -*- mode: c++; c-file-style: "linux"; c-basic-offset: 2; indent-tabs-mode: nil -*-
+// -*- mode: c++; c-file-style: "linux"; c-basic-offset: 2; indent-tabs-mode: nil -*-
 //
 //  Copyright (C) 2004-2015 Andrej Vodopivec <andrej.vodopivec@gmail.com>
 //            (C) 2012-2013 Doug Ilijev <doug.ilijev@gmail.com>
@@ -17,7 +17,7 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 //
 //  SPDX-License-Identifier: GPL-2.0+
 
@@ -30,6 +30,7 @@
 #ifndef FINDREPLACEPANE_H
 #define FINDREPLACEPANE_H
 
+#include "precomp.h"
 #include <wx/fdrepdlg.h>
 #include <wx/event.h>
 #include <wx/panel.h>
@@ -85,8 +86,6 @@ protected:
 
   void OnKeyDown(wxKeyEvent &event);
 
-DECLARE_EVENT_TABLE()
-
 };
 
 #endif // FINDREPLACEPANE_H
