@@ -15,55 +15,54 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 //
 //  SPDX-License-Identifier: GPL-2.0+
 
 #include "LogPane.h"
-
 LogPane::LogPane(wxWindow *parent, wxWindowID id, bool becomeLogTarget) : wxPanel(parent, id)
 {
-  m_isLogTarget = false;
-  m_errorRedirector = NULL;
-  m_logPanelTarget = NULL;
   wxBoxSizer *vbox  = new wxBoxSizer(wxVERTICAL);
 
   m_textCtrl = new wxTextCtrl(this, -1, wxEmptyString, wxDefaultPosition,
 					wxDefaultSize,
-					wxTE_MULTILINE | wxTE_READONLY | wxHSCROLL);
+					wxTE_MULTILINE | wxTE_READONLY | wxHSCROLL| wxVSCROLL);
 
-  vbox->Add(m_textCtrl, wxSizerFlags().Expand().Proportion(10));
-
-  SetSizerAndFit(vbox);
-  if(becomeLogTarget)
-    BecomeLogTarget();
+  m_textCtrl->SetMinSize(wxSize(wxSystemSettings::GetMetric( wxSYS_SCREEN_X )/10,
+                                wxSystemSettings::GetMetric( wxSYS_SCREEN_Y )/10));
+  vbox->Add(m_textCtrl, wxSizerFlags().Expand().Proportion(1));
+    
+  if (becomeLogTarget)
+    BecomeLogTarget();    
 
   // m_logPanelTarget->SetRepetitionCounting();
   // m_logPanelTarget->DisableTimestamp();
-  SetMinSize(wxSize(wxSystemSettings::GetMetric ( wxSYS_SCREEN_X )/10,
-                    wxSystemSettings::GetMetric ( wxSYS_SCREEN_Y )/10));
+  SetSizerAndFit(vbox);
 }
 
 void LogPane::DropLogTarget()
 {
-  // m_logPanelTarget is automatically destroyed in this step.
-  if(m_isLogTarget)
+  if (m_errorRedirector)
   {
-    wxDELETE(m_errorRedirector);
-    wxLog::SetActiveTarget(NULL);
+    m_errorRedirector.reset(); // redirector restores old target on destruction
+    wxLog::SetActiveTarget(nullptr); // but we don't want to be a target
   }
-  m_logPanelTarget = m_errorRedirector = NULL;
-  m_isLogTarget = false;
+  m_logPanelTarget.reset();
 }
 
 void LogPane::BecomeLogTarget()
 {
-  m_isLogTarget = true;
-  wxLog::SetActiveTarget(m_logPanelTarget = new wxLogTextCtrl(m_textCtrl));  
-  m_errorRedirector = new ErrorRedirector(new wxLogGui());
+  m_logPanelTarget.emplace(m_textCtrl);
+  wxLog::SetActiveTarget(&*m_logPanelTarget);
+  m_errorRedirector.emplace(std::unique_ptr<wxLog>(new wxLogGui()));
+  #ifdef wxUSE_STD_IOSTREAM
+  if (!ErrorRedirector::LoggingToStdErr())
+    m_textRedirector.emplace(m_textCtrl);
+  #endif
 }
 
 LogPane::~LogPane()
 {
   DropLogTarget();
 }
+
