@@ -15,7 +15,7 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 //
 //  SPDX-License-Identifier: GPL-2.0+
 
@@ -29,7 +29,7 @@ MainMenuBar::MainMenuBar() : wxMenuBar()
 {
 }
 
-void MainMenuBar::Enable(int id, bool enable)
+void MainMenuBar::EnableItem(int id, bool enable)
 {
   if(IsEnabled(id) != enable)
     wxMenuBar::Enable(id,enable);
