@@ -15,7 +15,7 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 //
 //  SPDX-License-Identifier: GPL-2.0+
 
@@ -26,18 +26,18 @@
 #ifndef MAINMENUBAR_H
 #define MAINMENUBAR_H
 
+#include "precomp.h"
 #include <wx/wx.h>
 #include <wx/menu.h>
 
-/*! A panel that shows an example image
+/*! A menu bar that flickers less on MS windows
 
-  From https://forums.wxwidgets.org/viewtopic.php?t=21664 with a few modifications.
  */
 class MainMenuBar : public wxMenuBar
 {
  public:
   MainMenuBar();
   //! Enable or disable an item - but only if the item needs enabling or disabling
-  void Enable(int id, bool enable);
+  void EnableItem(int id, bool enable);
 };
 #endif // MAINMENUBAR_H
