@@ -1,4 +1,4 @@
-﻿// -*- mode: c++; c-file-style: "linux"; c-basic-offset: 2; indent-tabs-mode: nil -*-
+// -*- mode: c++; c-file-style: "linux"; c-basic-offset: 2; indent-tabs-mode: nil -*-
 //
 //  Copyright (C) 2004-2015 Andrej Vodopivec <andrej.vodopivec@gmail.com>
 //  Copyright (C) 2017-2018 Gunter Königsmann <wxMaxima@physikbuch.de>
@@ -16,13 +16,14 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 //
 //  SPDX-License-Identifier: GPL-2.0+
 
 #ifndef MAXSIZECHOOSER_H
 #define MAXSIZECHOOSER_H
 
+#include "precomp.h"
 #include <wx/wx.h>
 #include <wx/statline.h>
 #include <wx/spinctrl.h>
@@ -41,7 +42,7 @@ public:
     return m_width->GetValue();
   }
 
-  double GetMaxHeight()
+  double GetHeightList()
   {
     return m_height->GetValue();
   }
