@@ -18,7 +18,7 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 //
 //  SPDX-License-Identifier: GPL-2.0+
 
@@ -88,7 +88,7 @@ bool Notification::Close()
 
 void Notification::OnClick(wxCommandEvent &WXUNUSED(event))
 {
-  if(GetGroup() != NULL)
+  if (GetGroup())
   {
     GetGroup()->Raise();
     GetGroup()->Show();
