@@ -1,6 +1,7 @@
-﻿// -*- mode: c++; c-file-style: "linux"; c-basic-offset: 2; indent-tabs-mode: nil -*-
+// -*- mode: c++; c-file-style: "linux"; c-basic-offset: 2; indent-tabs-mode: nil -*-
 //
 //  Copyright (C) 2006-2015 Andrej Vodopivec <andrej.vodopivec@gmail.com>
+//  Copyright (C) 2015-2019 Gunter Königsmann <wxMaxima@physikbuch.de>
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -15,7 +16,7 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 //
 
 //
@@ -27,17 +28,21 @@
 
 #include "TipOfTheDay.h"
 #include <wx/config.h>
-#include <wx/mstream.h>
 #include <wx/display.h>
-#include <wx/wfstream.h>
 #include <wx/persist.h>
 #include <wx/persist/toplevel.h>
+#include "SvgBitmap.h"
+#include <wx/mstream.h>
+#include <wx/wfstream.h>
+#include <wx/zstream.h>
+#include <wx/txtstrm.h>
+#include "Image.h"
 
 #define ICON_SCALE (0.35)
 #define ABS(val) ((val) >= 0 ? (val) : -(val))
 
 TipOfTheDay::TipOfTheDay(wxWindow *parent)
-  : wxDialog(parent,-1,_("Tip of the Day"))
+  : wxDialog(parent,-1,_("Tip of the day"))
 {
   m_tips.Add(
     _("To start using wxMaxima right away, start typing your command. An input cell should appear. Then press Shift-Enter to evaluate your command.")
@@ -136,7 +141,7 @@ TipOfTheDay::TipOfTheDay(wxWindow *parent)
     _("wxMaxima provides the \"table_form\" command that shows lists in an alternative form. One example would be:\n\ntable_form([1,2,3,4,5]);")
     );
   m_tips.Add(
-    _("In order to visualize how a parameter affects an equation wxmaxima provides commands that start with \"with_slider_\" and that create animations. One example would be:\n\n    with_slider_draw(\n        a,[-16,-9,-4,-2,0,1,2,3,4,5,6l7],\n        title=concat(\"a=\",i),\n        grid=true,\n        explicit(\n            x^2-a*x,\n            x,-10,10\n        )\n    );")
+    _("In order to visualize how a parameter affects an equation wxmaxima provides commands that start with \"with_slider_\" and that create animations. One example would be:\n\n    with_slider_draw(\n        a,[-16,-9,-4,-2,0,1,2,3,4,5,6,7],\n        title=concat(\"a=\",a),\n        grid=true,\n        explicit(\n            x^2-a*x,\n            x,-10,10\n        )\n    );")
     );
   m_tips.Add(
     _("If the \"Autosave\" functionality is enabled in the configuration dialogue wxMaxima acts like mobile apps that backup the file every few minutes and save it on closing.")
@@ -145,7 +150,7 @@ TipOfTheDay::TipOfTheDay(wxWindow *parent)
     _("It is possible to add custom symbols to the \"symbols\" sidebar by copying them into the respective field in the configuration dialogue.")
     );
   m_tips.Add(
-    _("Maxima's strengths are manipulating equations and in symbolic calculations. It therefore makes sense to use functions (as opposed to equations with labels) sparingly and to keep the actual values of variables in a list, instead of directly assigning them values. An example session that does do so would be:\n\n/* We keep the actual values in a list so we can use them lateron */\n    Values:[a=10,c=100];\n    Pyth:a^2+b^2=c^2;\n    solve(%,b);\n    result:%[2];\n    at(result,Values);\n    float(%);")
+    _("Maxima's strengths are manipulating equations and in symbolic calculations. It therefore makes sense to use functions (as opposed to equations with labels) sparingly and to keep the actual values of variables in a list, instead of directly assigning them values. An example session that does do so would be:\n\n/* We keep the actual values in a list so we can use them later on */\n    Values:[a=10,c=100];\n    Pyth:a^2+b^2=c^2;\n    solve(%,b);\n    result:%[2];\n    at(result,Values);\n    float(%);")
     );
   m_tips.Add(
     _("Equations have several advantages over functions. For example they can be manipulated with factor(), expand() and similar functions. They can easily be introduced one into another. Also they are always printed out as 2D maths.")
@@ -168,6 +173,12 @@ TipOfTheDay::TipOfTheDay(wxWindow *parent)
   m_tips.Add(
     _("The key combination Shift+Space results in a non-breakable space.")
     );
+  m_tips.Add(
+    _("wxMaxima needs more translators. We do not speak every language. Help us to translate wxMaxima and the manual to your language. Join the wxMaxima development at: https://github.com/wxMaxima-developers/wxmaxima")
+    );
+  m_tips.Add(
+    _("wxMaxima needs more developers. It is an open source project where you can contribute. Join the wxMaxima development at: https://github.com/wxMaxima-developers/wxmaxima")
+    );
 
   m_num = 0;
   wxConfigBase *config = wxConfig::Get();
@@ -176,7 +187,7 @@ TipOfTheDay::TipOfTheDay(wxWindow *parent)
     m_num = m_tips.GetCount()-1;
   if((unsigned)m_num >=m_tips.GetCount())
     m_num = 0;
-  
+
   SetName("TipOfTheDay");
   SetTitle(_("Tip of the day"));
   wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);
@@ -184,8 +195,7 @@ TipOfTheDay::TipOfTheDay(wxWindow *parent)
   wxButton *backButton = new wxButton(this,-1);
   backButton->SetBitmap(
     GetImage(
-      media_playback_start_reverse_128_png,media_playback_start_reverse_128_png_len,
-      media_playback_start_reverse_192_png,media_playback_start_reverse_192_png_len
+      media_playback_start_reverse_svg_gz,media_playback_start_reverse_svg_gz_len
       )
     );
   backButton->Connect(
@@ -201,8 +211,7 @@ TipOfTheDay::TipOfTheDay(wxWindow *parent)
   wxButton *forwardButton = new wxButton(this,-1);
   forwardButton->SetBitmap(
     GetImage(
-      media_playback_start_128_png,media_playback_start_128_png_len,
-      media_playback_start_192_png,media_playback_start_192_png_len
+      media_playback_start_svg_gz,media_playback_start_svg_gz_len
       )
     );
   forwardButton->Connect(
@@ -212,13 +221,13 @@ TipOfTheDay::TipOfTheDay(wxWindow *parent)
     );
   hbox->Add(forwardButton,wxSizerFlags().Expand());
   vbox->Add(hbox, wxSizerFlags().Expand());
-  
+
   m_tip = new wxTextCtrl(this,-1,m_tips[m_num],
                          wxDefaultPosition,wxDefaultSize,
                          wxTE_READONLY | wxTE_MULTILINE
     );
   vbox->Add(m_tip, wxSizerFlags().Expand().Proportion(10));
-  
+
   wxBoxSizer *buttonSizer = new wxBoxSizer(wxHORIZONTAL);
 
   m_showAtStartup = new wxCheckBox(this, -1, _("Show tips at Startup"));
@@ -236,11 +245,11 @@ TipOfTheDay::TipOfTheDay(wxWindow *parent)
     );
 
   buttonSizer->Add(okButton);
-  okButton->SetDefault(); 
+  okButton->SetDefault();
   vbox->Add(buttonSizer, wxSizerFlags().Expand().Proportion(1));
 
-  SetSizerAndFit(vbox);
   wxPersistenceManager::Get().RegisterAndRestore(this);
+  SetSizerAndFit(vbox);
   config->Write(wxT("tipNum"), m_num + 1);
 }
 
@@ -252,13 +261,12 @@ TipOfTheDay::~TipOfTheDay()
 }
 
 
-wxImage TipOfTheDay::GetImage(unsigned char *data_128, size_t len_128,
-                              unsigned char *data_192, size_t len_192)
+wxImage TipOfTheDay::GetImage(unsigned char *data, size_t len)
 {
   int ppi;
 #if wxCHECK_VERSION(3, 1, 1)
   wxDisplay display;
-  
+
   int display_idx = wxDisplay::GetFromWindow(GetParent());
   if (display_idx < 0)
     ppi = 72;
@@ -271,9 +279,8 @@ wxImage TipOfTheDay::GetImage(unsigned char *data_128, size_t len_128,
     ppi = wxGetDisplayPPI().x;
   if(ppi <= 10)
     ppi = 72;
-  
-  double targetSize = wxMax(ppi,75) * ICON_SCALE;
-  int prescale;
+
+  int targetSize = wxMax(ppi,75) * ICON_SCALE;
 
   int sizeA = 128 << 4;
   while(sizeA * 3 / 2 > targetSize && sizeA >= 32) {
@@ -287,40 +294,22 @@ wxImage TipOfTheDay::GetImage(unsigned char *data_128, size_t len_128,
 
   if(ABS(targetSize - sizeA) < ABS(targetSize - sizeB)) {
     targetSize = sizeA;
-    prescale = 128;
   } else {
     targetSize = sizeB;
-    prescale = 192;
   }
 
-  wxBitmap bmp;
-  wxImage img;
+  wxImage img = SvgBitmap(data, len, targetSize, targetSize).ConvertToImage();
 
-  void *data;
-  size_t len;
-  if(prescale == 128)
-  {
-    data = (void *)data_128;
-    len  = len_128;
-  }
-  else
-  {
-    data = (void *)data_192;
-    len  = len_192;
-  }
-  wxMemoryInputStream istream(data,len);
-  img.LoadFile(istream);
-  
 #if defined __WXMSW__
 #if wxCHECK_VERSION(3, 1, 1)
   // MSW is notorious for having problems with transparent black pixels.
   // Let's see if we can avoid these problems by converting the alpha
-  // channel to a mask even if that means we cannot antialias a transparent 
+  // channel to a mask even if that means we cannot antialias a transparent
   // and a colored pixel to a half-transparent one any more.
   img.ConvertAlphaToMask();
 #endif
 #endif
-  
+
   img.Rescale(targetSize, targetSize, wxIMAGE_QUALITY_HIGH);
   return img;
 }
