@@ -1,4 +1,4 @@
-﻿// -*- mode: c++; c-file-style: "linux"; c-basic-offset: 2; indent-tabs-mode: nil -*-
+// -*- mode: c++; c-file-style: "linux"; c-basic-offset: 2; indent-tabs-mode: nil -*-
 //
 //  Copyright (C) 2004-2015 Andrej Vodopivec <andrej.vodopivec@gmail.com>
 //            (C) 2012 Doug Ilijev <doug.ilijev@gmail.com>
@@ -17,7 +17,7 @@
 //
 //  You should have received a copy of the GNU General Public License
 //  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+//  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 //
 //  SPDX-License-Identifier: GPL-2.0+
 
@@ -30,6 +30,7 @@ surrounding the worksheet.
 #ifndef WXMAXIMAFRAME_H
 #define WXMAXIMAFRAME_H
 
+#include "precomp.h"
 #include <wx/wx.h>
 
 #include <wx/dirctrl.h>
@@ -39,16 +40,17 @@ surrounding the worksheet.
 #include <wx/arrstr.h>
 #include <wx/aui/aui.h>
 #include <wx/notifmsg.h>
+#include <wx/wrapsizer.h>
 
 #include "Worksheet.h"
 #include "RecentDocuments.h"
 #include "Version.h"
 #include "MainMenuBar.h"
 #include "History.h"
-#include "ToolBar.h"
 #include "XmlInspector.h"
 #include "StatusBar.h"
 #include "LogPane.h"
+#include "ButtonWrapSizer.h"
 #include <list>
 
 
@@ -60,11 +62,11 @@ public:
   wxMaximaFrame(wxWindow *parent, int id, const wxString &title,
                 const wxPoint &pos = wxDefaultPosition,
                 const wxSize &size = wxDefaultSize,
-                long style = wxDEFAULT_FRAME_STYLE, bool becomeLogTarget = true);
+                long style = wxDEFAULT_FRAME_STYLE | wxSYSTEM_MENU | wxCAPTION, bool becomeLogTarget = true);
 
   /*! The destructor
   */
-  ~wxMaximaFrame();
+  virtual ~wxMaximaFrame();
 
   /*! Shows or hides the toolbar
     \param show
@@ -103,6 +105,7 @@ public:
     menu_pane_xmlInspector, //!< Both the "toggle the xml monitor" command and the monitor pane
     menu_pane_format,    //!< Both the "toggle the format pane" command and the format pane
     menu_pane_greek,     //!< Both the "toggle the greek pane" command and the "greek" pane
+    menu_pane_unicode,   //!< Both the "toggle the unicode pane" command and the "unicode" pane
     menu_pane_log,       //!< Both the "toggle the log pane" command and the "log" pane
     menu_pane_variables, //!< Both the "toggle the variables pane" command and the "variables" pane
     menu_pane_draw,      //!< Both the "toggle the draw pane" command for the "draw" pane
@@ -113,14 +116,10 @@ public:
       that this entry stays that of the last pane in this enum.
     */
     menu_pane_stats,
-
+    menu_pane_dockAll,
     input_line_id,
     refresh_id,
-    menu_new_id,
-    menu_open_id,
     menu_batch_id,
-    menu_save_id,
-    menu_save_as_id,
     menu_load_id,
     menu_sconsole_id,
     menu_interrupt_id,
@@ -140,9 +139,23 @@ public:
     menu_gen_mat,
     menu_gen_mat_lambda,
     menu_enter_mat,
+    menu_csv2mat,
+    menu_mat2csv,
+    menu_csv2list,
+    menu_list2csv,
+    menu_matrix_row,
+    menu_matrix_col,
+    menu_matrix_row_list,
+    menu_matrix_col_list,
+    menu_submatrix,
+    menu_matrix_multiply,
+    menu_matrix_exponent,
+    menu_matrix_hadamard_product,
+    menu_matrix_hadamard_exponent,
     menu_invert_mat,
     menu_cpoly,
     menu_determinant,
+    menu_rank,
     menu_eigen,
     menu_eigvect,
     menu_fun_def,
@@ -166,9 +179,11 @@ public:
     menu_demoivre,
     menu_exponentialize,
     menu_num_out,
+    menu_stringdisp,
     menu_to_float,
     menu_to_bfloat,
     menu_to_numer,
+    menu_num_domain,
     menu_set_precision,
     menu_set_displayprecision,
     menu_engineeringFormat,
@@ -197,6 +212,7 @@ public:
     menu_atvalue,
     menu_lhs,
     menu_rhs,
+    menu_wxmaximahelp,
     menu_maximahelp,
     menu_example,
     menu_apropos,
@@ -216,9 +232,14 @@ public:
     menu_noAutosubscript,
     menu_defaultAutosubscript,
     menu_alwaysAutosubscript,
+    menu_autosubscriptIndividual,
+    menu_declareAutosubscript,
+    menu_noAutosubscriptIndividual,
     menu_roundedMatrixParens,
-    menu_roundedMatrixParensYes,
-    menu_roundedMatrixParensNo,
+    menu_squareMatrixParens,
+    menu_straightMatrixParens,
+    menu_angledMatrixParens,
+    menu_noMatrixParens,
     menu_draw_2d,
     menu_draw_3d,
     menu_draw_explicit,
@@ -233,6 +254,7 @@ public:
     menu_draw_axis,
     menu_draw_accuracy,
     menu_draw_contour,
+    menu_license,
     button_factor_id,
     button_solve,
     button_solve_ode,
@@ -269,13 +291,9 @@ public:
     menu_evaluate_all_visible,
     menu_evaluate_all,
     menu_show_tip,
-    menu_copy_from_worksheet,
-	menu_copy_matlab_from_worksheet,
+    menu_copy_matlab_from_worksheet,
     menu_copy_tex_from_worksheet,
     menu_copy_text_from_worksheet,
-    menu_undo,
-    menu_redo,
-    menu_select_all,
     menu_logcontract,
     menu_logexpand,
     menu_to_fact,
@@ -283,7 +301,7 @@ public:
     menu_texform,
     button_enter,
     menu_zoom_80,
-    menu_zoom_100,
+    /* Instead of menu_zoom_100 we use the standard wxID_ZOOM_100, which displays an icon in the menu (currently Unix only) */
     menu_zoom_120,
     menu_zoom_150,
     menu_zoom_200,
@@ -322,14 +340,9 @@ public:
     menu_insert_previous_output,
     menu_autocomplete,
     menu_autocomplete_templates,
-    menu_cut,
-    menu_paste,
     menu_paste_input,
     menu_fullscreen,
     menu_remove_output,
-    mac_newId,
-    mac_openId,
-    mac_closeId,
     menu_list_create_from_elements,
     menu_list_create_from_rule,
     menu_list_create_from_list,
@@ -355,7 +368,8 @@ public:
     menu_list_as_function_arguments,
     menu_list_do_for_each_element,
     menu_list_remove_element,
-    menu_list_append_item,
+    menu_list_append_item_start,
+    menu_list_append_item_end,
     menu_list_append_list,
     menu_list_interleave,
     menu_recent_packages,
@@ -480,15 +494,31 @@ public:
     menu_format_image,
     menu_format_pagebreak,
     menu_help_tutorials,
+    menu_help_tutorials_start, //! Start of bundled tutorials
+    menu_help_solving,
+    menu_help_diffequations,
+    menu_help_numberformats,
+    menu_help_tolerances,
+    menu_help_listaccess,
+    menu_help_memoizing,
+    menu_help_3d,
+    menu_help_varnames,
+    menu_help_fittingData,
+    menu_help_tutorials_end, //! End of bundled tutorials
     menu_show_toolbar,
-    menu_edit_find,
     menu_history_previous,
     menu_history_next,
     menu_check_updates,
     socket_client_id,
     socket_server_id,
     maxima_process_id,
-    gnuplot_process_id
+    gnuplot_query_terminals_id,
+    gnuplot_process_id,
+    menu_additionalSymbols,
+    enable_unicodePane,
+    menu_showLatinGreekLookalikes,
+    menu_showGreekMu,
+    menu_invertWorksheetBackground
   };
 
   /*! Update the recent documents list
@@ -508,14 +538,14 @@ public:
   /*! Show or hide a sidebar
     
     \param id The type of the sidebar to show/hide
-    \param hide 
+    \param show 
      - true: show the sidebar
      - false: hide it
    */
-  void ShowPane(Event id, bool hide);
+  void ShowPane(Event id, bool show = true);
 
   //! Adds a command to the list  of recently used maxima commands
-  void AddToHistory(wxString cmd)
+  void AddToHistory(const wxString &cmd)
   { m_history->AddToHistory(cmd); }
 
   enum ToolbarStatus
@@ -573,8 +603,6 @@ protected:
   long m_bytesFromMaxima;
   //! The process id of maxima. Is determined by ReadFirstPrompt.
   long m_pid;
-  //! Did the user ever give this file a name?
-  bool m_isNamed;
   //! The last name GetTempAutosavefileName() has returned.
   wxString m_tempfileName;
   //! Issued if a notification is closed.
@@ -621,23 +649,19 @@ protected:
   void RemoveTempAutosavefile();
   //! Re-read the configuration.
   void ReReadConfig();  
-  /*! Determine a suitable name for a temporary autosave file.
-    
-    Is used if we want to autosave the current file, but still have no 
-    filename to save it to.
-  */  
-  wxString GetTempAutosavefileName();
-  //! Remember an temporary autosave file name.
+  //! Remember a temporary autosave file name.
   void RegisterAutoSaveFile();
+  /*! An instant single-window mode
+
+    A last resort if https://trac.wxwidgets.org/ticket/18815 hinders one from 
+    re-docking windows.
+   */
+  void DockAllSidebars(wxCommandEvent &ev);
+
 private:
   //! How many bytes did maxima send us when we updated the statusbar?
   long m_bytesFromMaxima_last;
   wxTimer m_bytesReadDisplayTimer; 
-  //! A panel that shows all user-defined symbols on the symbols pane.
-  wxPanel *m_userSymbols;
-  //! A button per user defined symbol
-  std::list<wxPanel *> m_userSymbolButtons;
-  wxGridSizer *m_userSymbolsSizer;
   //! True=We are currently saving.
   bool m_StatusSaving;
 
@@ -648,17 +672,17 @@ private:
 */
   void SetupMenu();
 
-  wxPanel *CreateStatPane();
+  wxWindow *CreateStatPane();
 
-  wxPanel *CreateMathPane();
-
-  wxPanel *CreateFormatPane();
+  wxWindow *CreateMathPane();
 
+  wxWindow *CreateFormatPane();
+  
   //! The class for the sidebar with the draw commands
-  class DrawPane: public wxPanel
+  class DrawPane: public wxScrolled<wxPanel>
     {
     public:
-      DrawPane(wxWindow *parent, int ID = wxID_ANY);
+      explicit DrawPane(wxWindow *parent, int id = wxID_ANY);
       /*! Tell the sidebar if we currently are inside a 2D or a 3D plot command
         
         \param dimensions
@@ -668,7 +692,10 @@ private:
       */
       void SetDimensions(int dimensions);
       int  GetDimensions() { return m_dimensions; }
+    protected:
+      void OnSize(wxSizeEvent &event);
     private:
+      Buttonwrapsizer *m_grid;
       wxButton *m_draw_setup2d;
       wxButton *m_draw_setup3d;
       wxButton *m_draw_explicit;
@@ -686,11 +713,13 @@ private:
       int m_dimensions;
     };
 public:
-  void LeftStatusText(wxString text, bool saveInLog = true)
+  void LeftStatusText(const wxString &text, bool saveInLog = true)
     {m_newLeftStatusText = true; m_leftStatusText = text; if(saveInLog)wxLogMessage(text);}
-  void RightStatusText(wxString text, bool saveInLog = true)
+  void RightStatusText(const wxString &text, bool saveInLog = true)
     {m_newRightStatusText = true; m_rightStatusText = text; if(saveInLog)wxLogMessage(text);}
 protected:
+  //! Are we inside a 2d or 3d draw command?
+  long m_drawDimensions_last;
   //! Do we have new text to output in the Right half of the Status Bar?
   bool m_newRightStatusText;
   //! Do we have new text to output in the Left half of the Status Bar?
@@ -704,26 +733,48 @@ protected:
   //! The sidebar with the draw commands
   DrawPane *m_drawPane;
 private:
-  /*! A flat, compact button for the greek and the symbols pane
-    
-    \param parent The parent panel/window
-    \param ch The unicode symbol
-    \param description The help text for the symbol
-    \param matchesMaximaCommand true means that this symbol is automatically
-                                translated into a maxima command/operator
+  class GreekPane : public wxScrolled<wxPanel>
+  {
+  public:
+    GreekPane(wxWindow *parent, Configuration *configuration, Worksheet *worksheet, int ID = wxID_ANY);
+  protected:
+    void UpdateSymbols();
+    void OnMouseRightDown(wxMouseEvent &event);
+    void OnMenu(wxCommandEvent &event);
+    void OnSize(wxSizeEvent &event);
+  private:
+    Configuration *m_configuration;
+    wxSizer *m_lowercaseSizer;
+    wxSizer *m_uppercaseSizer;
+    Worksheet *m_worksheet;
+  };
 
-   */
-  wxPanel *CharButton(wxPanel *parent, wxChar ch, wxString description = wxEmptyString, bool matchesMaximaCommand = false);
-
-  wxPanel *CreateGreekPane();
+  class SymbolsPane : public wxScrolled<wxPanel>
+  {
+  public:
+    SymbolsPane(wxWindow *parent, Configuration *configuration, Worksheet *worksheet, int ID = wxID_ANY);
+    //! Update the "user symbols" portion of the symbols pane.
+    void UpdateUserSymbols();
+  protected:
+    void OnMouseRightDown(wxMouseEvent &event);
+    void OnMenu(wxCommandEvent &event);
+    void OnSize(wxSizeEvent &event);
+  private:
+    //! A panel that shows all user-defined symbols on the symbols pane.
+    wxPanel *m_userSymbols;
+    //! A button per user defined symbol
+    std::list<wxWindow *> m_userSymbolButtons;
+    wxSizer *m_userSymbolsSizer;
+    Configuration *m_configuration;
+    Worksheet *m_worksheet;
+  };
 
   wxPanel *CreateSymbolsPane();
 
 protected:
+  SymbolsPane *m_symbolsPane;
   //! The current length of the evaluation queue of commands we still need to send to maxima
   int m_EvaluationQueueLength;
-  //! Update the "user symbols" portion of the symbols pane.
-  void UpdateUserSymbols();
   //! Do we need to update the display showing the evaluation queue length?
   bool m_updateEvaluationQueueLengthDisplay;
   //! The number of commands left in the current of the evaluation queue item
@@ -732,10 +783,7 @@ protected:
   //! Do we expect the 1st prompt from maxima to appear?
   bool m_first;
 
-  void CharacterButtonPressed(wxMouseEvent &event);
-
   bool ToolbarIsShown();
-
   //! The manager for dynamic screen layouts
   wxAuiManager m_manager;
   //! A XmlInspector-like xml monitor
@@ -753,6 +801,9 @@ protected:
   RecentDocuments m_recentPackages;
   wxMenu *m_recentDocumentsMenu;
   wxMenu *m_recentPackagesMenu;
+  wxMenu *m_autoSubscriptMenu;
+  wxMenu *m_equationTypeMenuMenu;
+  wxMenu *m_roundedMatrixParensMenu;
 };
 
 #endif // WXMAXIMAFRAME_H
